var __rest = this && this.__rest || function (s, e) {
  var t = {};

  for (var p in s) {
    if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];
  }

  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
    if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i])) t[p[i]] = s[p[i]];
  }
  return t;
};

import React, { useContext } from 'react';
import classNames from 'classnames';
import { Field } from 'rc-field-form';
import FieldContext from 'rc-field-form/lib/FieldContext';
import { FormContext } from './context';
import { toArray } from './utils';
import List from '../list';
var classPrefix = "am-form-item";
var MemoInput = /*#__PURE__*/React.memo(function (_ref) {
  var children = _ref.children;
  return children;
}, function (prev, next) {
  return prev.value === next.value && prev.update === next.update;
});

var FormItemLayout = function FormItemLayout(props) {
  var _classNames;

  var className = props.className,
      style = props.style,
      label = props.label,
      help = props.help,
      required = props.required,
      disabled = props.disabled,
      meta = props.meta,
      children = props.children,
      htmlFor = props.htmlFor;
  var context = useContext(FormContext);
  var hasFeedback = props.hasFeedback || context.hasFeedback;
  var layout = props.layout || context.layout;
  var formItemLabelClass = classNames(classPrefix + "-label", (_classNames = {}, _classNames[classPrefix + "-label-disable"] = disabled, _classNames));
  var feedback = hasFeedback && meta && meta.errors.length > 0 ? meta.errors[0] : null;
  var labelElement = /*#__PURE__*/React.createElement("label", {
    className: formItemLabelClass,
    htmlFor: htmlFor
  }, required && /*#__PURE__*/React.createElement("span", {
    className: classPrefix + "-label-required"
  }, "*"), label, help && /*#__PURE__*/React.createElement("span", {
    className: classPrefix + "-label-help"
  }, help));
  var descriptionElement = feedback && /*#__PURE__*/React.createElement("div", {
    className: classPrefix + "-footer"
  }, feedback);
  return /*#__PURE__*/React.createElement(List.Item, {
    style: style,
    title: layout === 'vertical' && labelElement,
    prefix: layout === 'horizontal' && labelElement,
    description: descriptionElement,
    className: classNames(classPrefix, className),
    onClick: props.onClick
  }, children);
};

export var FormItem = function FormItem(props) {
  var className = props.className,
      style = props.style,
      displayLabel = props.label,
      help = props.help,
      hasFeedback = props.hasFeedback,
      name = props.name,
      required = props.required,
      noStyle = props.noStyle,
      disabled = props.disabled,
      rules = props.rules,
      children = props.children,
      messageVariables = props.messageVariables,
      _props$trigger = props.trigger,
      trigger = _props$trigger === void 0 ? 'onChange' : _props$trigger,
      validateTrigger = props.validateTrigger,
      onClick = props.onClick,
      fieldProps = __rest(props, ["className", "style", "label", "help", "hasFeedback", "name", "required", "noStyle", "disabled", "rules", "children", "messageVariables", "trigger", "validateTrigger", "onClick"]);

  var _React$useContext = React.useContext(FieldContext),
      contextValidateTrigger = _React$useContext.validateTrigger;

  var mergedValidateTrigger = validateTrigger !== undefined ? validateTrigger : contextValidateTrigger;
  var updateRef = React.useRef(0);
  updateRef.current += 1; // 如果 label 未设置，那么尝试使用 name 作为 label

  var label = displayLabel ? displayLabel : typeof name === 'string' ? name : undefined;

  function renderLayout(baseChildren, fieldId, meta, isRequired) {
    if (noStyle) {
      return baseChildren;
    }

    return /*#__PURE__*/React.createElement(FormItemLayout, {
      className: className,
      style: style,
      label: label,
      help: help,
      required: isRequired,
      disabled: disabled,
      hasFeedback: hasFeedback,
      htmlFor: fieldId,
      meta: meta,
      onClick: onClick
    }, baseChildren);
  }

  var isRenderProps = typeof children === 'function';

  if (!name && !isRenderProps && !props.dependencies) {
    return renderLayout(children);
  }

  var variables = {};

  if (typeof label === 'string') {
    variables.label = label;
  }

  if (messageVariables) {
    variables = Object.assign(Object.assign({}, variables), messageVariables);
  }

  return /*#__PURE__*/React.createElement(Field, Object.assign({}, fieldProps, {
    name: name,
    rules: rules,
    trigger: trigger,
    validateTrigger: mergedValidateTrigger
  }), function (control, meta, context) {
    var childNode = null;
    var isRequired = required !== undefined ? required : !!(rules && rules.some(function (rule) {
      if (rule && typeof rule === 'object' && rule.required) {
        return true;
      }

      return false;
    }));
    var fieldId = (toArray(name).length && meta ? meta.name : []).join('_');

    if (isRenderProps && (props.shouldUpdate || props.dependencies) && !name) {
      childNode = children(context);
    } else if ( /*#__PURE__*/React.isValidElement(children)) {
      var childProps = Object.assign(Object.assign({}, children.props), control);

      if (!childProps.id) {
        childProps.id = fieldId;
      }

      if (disabled) {
        childProps.disabled = disabled;
      } // We should keep user origin event handler


      var triggers = new Set([].concat(toArray(trigger), toArray(mergedValidateTrigger)));
      triggers.forEach(function (eventName) {
        childProps[eventName] = function () {
          var _a2, _c2;

          var _a, _b, _c;

          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          (_a = control[eventName]) === null || _a === void 0 ? void 0 : (_a2 = _a).call.apply(_a2, [control].concat(args));
          (_c = (_b = children.props)[eventName]) === null || _c === void 0 ? void 0 : (_c2 = _c).call.apply(_c2, [_b].concat(args));
        };
      });
      childNode = /*#__PURE__*/React.createElement(MemoInput, {
        value: control[props.valuePropName || 'value'],
        update: updateRef.current
      }, /*#__PURE__*/React.cloneElement(children, childProps));
    } else {
      childNode = children;
    }

    return renderLayout(childNode, fieldId, meta, isRequired);
  });
};