import { useInitialized } from '../../utils/use-initialized';
import { disableBodyScroll, enableBodyScroll } from 'body-scroll-lock';
import classNames from 'classnames';
import React, { useEffect, useRef } from 'react';
import { createPortal } from 'react-dom';
import { CSSTransition } from 'react-transition-group';
var classPrefix = "am-mask";
export var Mask = function Mask(props) {
  var _classNames;

  var cls = classNames(classPrefix, props.className, (_classNames = {}, _classNames[classPrefix + "-hidden"] = !props.visible, _classNames));
  var initialized = useInitialized(props.visible || props.forceRender);
  var ref = useRef(null); // 禁止 body 滚动 https://github.com/willmcpo/body-scroll-lock

  useEffect(function () {
    var element = ref.current;

    if (props.visible && props.disableBodyScroll) {
      disableBodyScroll(element);
      return function () {
        enableBodyScroll(element);
      };
    }
  }, [props.visible, props.disableBodyScroll]);

  function handleClick(e) {
    var _a;

    if (e.currentTarget === e.target) {
      (_a = props.onMaskClick) === null || _a === void 0 ? void 0 : _a.call(props, e);
    }
  }

  var opacity = props.opacity === 'default' ? 0.55 : props.opacity === 'dark' ? 0.75 : props.opacity;
  var node = /*#__PURE__*/React.createElement(CSSTransition, {
    "in": props.visible,
    timeout: 200,
    classNames: classPrefix,
    onExited: props.afterClose,
    unmountOnExit: props.destroyOnClose
  }, /*#__PURE__*/React.createElement("div", {
    className: cls,
    onClick: handleClick,
    ref: ref,
    style: Object.assign(Object.assign({}, props.style), {
      backgroundColor: "rgba(0, 0, 0, " + opacity + ")"
    })
  }, initialized && props.children));

  if (props.getContainer) {
    var container = typeof props.getContainer === 'function' ? props.getContainer() : props.getContainer;
    return /*#__PURE__*/createPortal(node, container);
  }

  return node;
};
Mask.defaultProps = {
  visible: true,
  destroyOnClose: false,
  forceRender: false,
  opacity: 'default',
  disableBodyScroll: true
};