import React, { forwardRef, useImperativeHandle } from 'react';
import Tooltip from 'rc-tooltip';
import { useControllableValue } from 'ahooks';
import classNames from 'classnames';
var classPrefix = "am-popover";
var enterClassName = 'entering';
var leaveClassName = 'leaving';
export var Popover = /*#__PURE__*/forwardRef(function (props, ref) {
  var _props$mode = props.mode,
      mode = _props$mode === void 0 ? 'light' : _props$mode;

  var _useControllableValue = useControllableValue(props, {
    valuePropName: 'visible',
    trigger: 'onVisibleChange',
    defaultValuePropName: 'defaultVisible',
    defaultValue: false
  }),
      visible = _useControllableValue[0],
      onVisibleChange = _useControllableValue[1];

  useImperativeHandle(ref, function () {
    return {
      show: function show() {
        return onVisibleChange(true);
      },
      hide: function hide() {
        return onVisibleChange(false);
      },
      visible: visible
    };
  }, [visible]);
  return /*#__PURE__*/React.createElement(Tooltip, Object.assign({}, props, {
    overlayClassName: classNames(classPrefix + "-" + mode, props.overlayClassName),
    destroyTooltipOnHide: props.destroyOnHide,
    prefixCls: classPrefix,
    getTooltipContainer: props.getContainer || function () {
      return document.body;
    },
    visible: visible,
    onVisibleChange: onVisibleChange,
    trigger: 'click',
    motion: {
      motionName: {
        appear: enterClassName,
        appearActive: enterClassName,
        enter: enterClassName,
        enterActive: enterClassName,
        leaveActive: leaveClassName,
        leave: leaveClassName
      },
      motionDeadline: 200
    },
    overlay: /*#__PURE__*/React.createElement("div", {
      className: classPrefix + "-inner-content"
    }, props.content)
  }), props.children);
});