import React, { useMemo, useCallback } from 'react';
import Picker from '../picker';
import { useControllableValue } from 'ahooks';
import dayjs from 'dayjs';
import { generateIntArray } from '../../utils/generate-int-array';
import { withDefaultProps } from '../../utils/with-default-props';
var thisYear = new Date().getFullYear();
var defaultProps = {
  min: new Date(new Date().setFullYear(thisYear - 10)),
  max: new Date(new Date().setFullYear(thisYear + 10))
};
export var DatePicker = withDefaultProps(defaultProps)(function (props) {
  var _useControllableValue = useControllableValue(props, {
    trigger: 'onConfirm',
    defaultValue: null
  }),
      value = _useControllableValue[0],
      setValue = _useControllableValue[1];

  function columns(selected) {
    var years = [];
    var minYear = props.min.getFullYear();
    var minMonth = props.min.getMonth() + 1;
    var minDate = props.min.getDate();
    var maxYear = props.max.getFullYear();
    var maxMonth = props.max.getMonth() + 1;
    var maxDate = props.max.getDate();
    var firstDayInSelectedMonth = dayjs(convertStringArrayToDate([selected[0], selected[1], '1']));
    var selectedYear = firstDayInSelectedMonth.year();
    var selectedMonth = firstDayInSelectedMonth.month() + 1;

    for (var i = minYear; i <= maxYear; i++) {
      years.push(i.toString());
    }

    var months = generateIntArray(1, 12).filter(function (v) {
      if (selectedYear === minYear && v < minMonth) {
        return false;
      }

      if (selectedYear === maxYear && v > maxMonth) {
        return false;
      }

      return true;
    }).map(function (v) {
      return v.toString();
    });
    var days = [];
    days = generateIntArray(1, firstDayInSelectedMonth.daysInMonth()).filter(function (v) {
      if (selectedYear === minYear && selectedMonth === minMonth && v < minDate) {
        return false;
      }

      if (selectedYear === maxYear && selectedMonth === maxMonth && v > maxDate) {
        return false;
      }

      return true;
    }).map(function (v) {
      return v.toString();
    });
    return [years, months, days];
  }

  var pickerValue = useMemo(function () {
    return convertDateToStringArray(value);
  }, [value]);
  var onConfirm = useCallback(function (val) {
    setValue(convertStringArrayToDate(val));
  }, [setValue]);
  var onSelect = useCallback(function (val) {
    var _a;

    var date = convertStringArrayToDate(val);

    if (date) {
      (_a = props.onSelect) === null || _a === void 0 ? void 0 : _a.call(props, date);
    }
  }, [props.onSelect]);
  return /*#__PURE__*/React.createElement(Picker, {
    columns: columns,
    value: pickerValue,
    onCancel: props.onCancel,
    onClose: props.onClose,
    visible: props.visible,
    confirmText: props.confirmText,
    cancelText: props.cancelText,
    onConfirm: onConfirm,
    onSelect: onSelect,
    getContainer: props.getContainer,
    afterShow: props.afterShow,
    afterClose: props.afterClose,
    onClick: props.onClick
  }, function (items) {
    var _a;

    return (_a = props.children) === null || _a === void 0 ? void 0 : _a.call(props, convertStringArrayToDate(items.map(function (item) {
      return item === null || item === void 0 ? void 0 : item.value;
    })));
  });
});

function convertDateToStringArray(date) {
  if (!date) return [];
  return [date.getFullYear().toString(), (date.getMonth() + 1).toString(), date.getDate().toString()];
}

function convertStringArrayToDate(value) {
  var _a, _b, _c;

  if (value.length !== 3) return null;
  var yearString = (_a = value[0]) !== null && _a !== void 0 ? _a : '1900';
  var monthString = (_b = value[1]) !== null && _b !== void 0 ? _b : '1';
  var dateString = (_c = value[2]) !== null && _c !== void 0 ? _c : '1';
  return new Date(parseInt(yearString), parseInt(monthString) - 1, parseInt(dateString));
}